import { types } from "@babel/core";
import { declare } from "@babel/helper-plugin-utils";
import * as THREE from "three";
const imports = /* @__PURE__ */ new Map();
let lastImport = null;
const index = declare((api) => {
  api.assertVersion(7);
  return {
    manipulateOptions(_, options) {
      options.plugins.push("jsx");
    },
    post() {
      imports.clear();
      lastImport = null;
    },
    visitor: {
      CallExpression(path) {
        if (types.isIdentifier(path.node.callee) && path.node.callee.name === "extend" && types.isIdentifier(path.node.arguments[0]) && path.node.arguments[0].name === "THREE") {
          path.remove();
        }
      },
      ImportDeclaration(importPath) {
        lastImport = importPath;
      },
      JSXOpeningElement(path) {
        var _a;
        const { name } = path.node;
        if (types.isJSXNamespacedName(name))
          return;
        let type = "property" in name ? name.property.name : name.name;
        const declaration = (_a = path.scope.getBinding(type)) == null ? void 0 : _a.path.node;
        if (types.isVariableDeclarator(declaration)) {
          if (types.isStringLiteral(declaration.init)) {
            type = declaration.init.value;
          } else if (types.isTemplateLiteral(declaration.init)) {
            type = declaration.init.quasis.map((n) => n.value.cooked).join("\\w*");
          } else if (types.isBinaryExpression(declaration.init) || types.isAssignmentExpression(declaration.init)) {
            type = [declaration.init.left, declaration.init.right].map((o) => types.isStringLiteral(o) ? o.value : "\\w*").join("");
          } else if (types.isConditionalExpression(declaration.init)) {
            type = [declaration.init.consequent, declaration.init.alternate].map((o) => types.isStringLiteral(o) ? o.value : "\\w*").join("|");
          } else {
            type = "\\w+";
          }
        }
        const pattern = new RegExp(`^${type}$`);
        for (const className in THREE) {
          const type2 = className.replace(/^[A-Z]/, (c) => c.toLowerCase());
          if (!imports.has(className) && pattern.test(type2)) {
            const local = path.scope.generateUidIdentifier(className);
            imports.set(className, types.importSpecifier(local, types.identifier(className)));
          }
        }
      },
      Program: {
        exit(path) {
          if (!imports.size)
            return;
          const THREEImports = types.importDeclaration([...imports.values()], types.stringLiteral("three"));
          const extendImport = types.importSpecifier(path.scope.generateUidIdentifier("extend"), types.identifier("extend"));
          const extendCall = types.expressionStatement(
            types.callExpression(extendImport.local, [
              types.objectExpression(
                THREEImports.specifiers.map(
                  (importSpecifier) => types.objectProperty(importSpecifier.imported, importSpecifier.local, false, true)
                )
              )
            ])
          );
          const R3FExports = types.importDeclaration([extendImport], types.stringLiteral("@react-three/fiber"));
          for (const node of [extendCall, R3FExports, THREEImports]) {
            if (lastImport)
              lastImport.insertAfter(node);
            else
              path.unshiftContainer("body", node);
          }
        }
      }
    }
  };
});
export {
  index as default
};
//# sourceMappingURL=index.mjs.map
